$regfile = "m32def.dat"
$crystal = 8000000

Const Tmr_c_prescale = 64
Const Tmr_c_preload = 131

$baud = 9600
$hwstack = 64
$swstack = 256
$framesize = 64


Const Max_c_unit = 16
Const Max_c_msg = 24

Const Rx_bufsize = 48

$include "myrncom.bas"
$include "myunit.bas"
$include "mymsgque.bas"


'-------------------------------------------------
'           Common Commands
'-------------------------------------------------
Const Cmd_nop = 0
Const Cmd_setup = 1
Const Cmd_init = 2

Const Cmd_time = 4
Const Cmd_set = 5
Const Cmd_msg = 16


'-------------------------------------------------
'           RNBFRA I2C
'-------------------------------------------------
Config I2cdelay = 5
Config Scl = Portc.0                              'Ports fuer IIC-Bus
Config Sda = Portc.1
'-------------------------------------------------
'           RNBFRA I2C Addresses
'-------------------------------------------------
Const Pwr_adr = &H74                              'I2C Adress Powerport
Const Out_adr = &H72                              'I2C Adress Extension-Out Port
Const Exp_adr = &H7E                              'I2C Read Adress Expansion-In Port


Const Pwr_default = &HFF                          'I2C Powerport
Const Out_default = &H00                          'I2C Extension-Out Port

'-------------------------------------------------
'           Class IDs
'-------------------------------------------------
Const Cls_pcf = &H42                              '
Const Cls_adc = &H82                              '


'-------------------------------------------------
'           CLASS entries     R8:R9=Flash Param  (restore indiv.params)
'-------------------------------------------------
Declare Sub Pcfclass(byval Tag As Word , Byval Cmd As Byte , Byval Param As Word)
Declare Sub Adcclass(byval Tag As Word , Byval Cmd As Byte , Byval Param As Word)


'=========================================================================================
' Timer0 Prescale for 1mS Interrupt
'=========================================================================================
Config Timer0 = Timer , Prescale = Tmr_c_prescale 'Timer 1mS
On Timer0 Interrupt_ticker                        ' Timer for Timer Queue

'-------------------------------------------------
' frequently used units
'-------------------------------------------------
Dim Pwrtag As Word
Dim Outtag As Word
Dim Exptag As Word

'-------------------------------------------------
'           work data
'-------------------------------------------------
Dim Par_byte As Byte
Dim Par_word As Word
Dim Par_addr As Word
Dim Mytag As Word
Dim Tempb As Byte

'-------------------------------------------------
'   enable
'-------------------------------------------------
   Enable Timer0
   Enable Interrupts


'-------------------------------------------------
'           setup Units
'-------------------------------------------------
' RNBFRA PCF Ports

      Exptag = Unit_new()
      Par_addr = Loadlabel(exp_par)               ' PCF Inp-Port
      Call Pcfclass(exptag , Cmd_setup , Par_addr)

      Outtag = Unit_new()
      Par_addr = Loadlabel(out_par)               ' PCF OUT-Port
      Call Pcfclass(outtag , Cmd_setup , Par_addr)

      Pwrtag = Unit_new()
      Par_addr = Loadlabel(pwr_par)               ' PCF PWR-Port
      Call Pcfclass(pwrtag , Cmd_setup , Par_addr)

      Par_addr = Loadlabel(adc_par0)              ' ADC 0
      Gosub Build_adc
      Par_addr = Loadlabel(adc_par1)              ' ADC 1
      Gosub Build_adc
      Par_addr = Loadlabel(adc_par2)              ' ADC 2
      Gosub Build_adc
      Par_addr = Loadlabel(adc_par3)              ' ADC 3
      Gosub Build_adc
      Par_addr = Loadlabel(adc_par4)              ' ADC 4
      Gosub Build_adc
      Par_addr = Loadlabel(adc_par5)              ' ADC 5
      Gosub Build_adc
      Par_addr = Loadlabel(adc_par6)              ' ADC 6
      Gosub Build_adc
      Par_addr = Loadlabel(adc_par7)              ' ADC 7
      Gosub Build_adc


'-------------------------------------------------
'           main loop
'-------------------------------------------------
   Do
      Gosub Rxinput
      Call Msg_dequeue(0 , 0 , 0)
   Loop

End
'==============================================================================
Build_adc:
      Mytag = Unit_new()
      Call Adcclass(mytag , Cmd_setup , Par_addr)
      Return
'==============================================================================
' Timer 0  interrupt
'==============================================================================
Interrupt_ticker:
   Timer0 = Tmr_c_preload
   Gosub Time_scan
Return

'==============================================================================
' Workout Frame
'==============================================================================
Dim Class As Byte                                 'destination Class
Dim Ident As Byte                                 'destination Ident
Dim Msg As Word                                   'Message-Base
Dim Leng As Byte

Rxinput:
   Msg = Com_msg()
   If Msg <> 0 Then
      Leng = Com_rdbyte()
      Class = Com_rdbyte()
      Ident = Com_rdbyte()
      Mytag = Unit_find_clsid(class , Ident)
      If Mytag <> 0 Then
            Call Unit_call(mytag , Cmd_msg , 0)
      Else
            Call Com_txstart()
            Call Com_txbyte(class)                ' dest class
            Call Com_txbyte(ident)                ' ident
            Leng = Leng - 3
            While Leng <> 0
               Tempb = Com_rdbyte()
               Call Com_txbyte(tempb)             ' source class
               Decr Leng
            Wend
            Call Com_txclose()
      End If
      Call Com_msgdone(msg)
   End If
   Return

'------------------------------------------------------------------------------
'        UNITS
'------------------------------------------------------------------------------
'------------------------------------------------------------------------------
'        Class PCF
'        private class data
'------------------------------------------------------------------------------
Const Max_c_pcf = 3
Dim Pcfidx As Byte

Dim Pcf_bck(max_c_pcf) As Word                    ' backlink to tag
Dim Pcf_value(max_c_pcf) As Byte                  ' private data

'-----------------------------------------------
'        Class parameter
'-----------------------------------------------
'         class   idn  devadr    default      timeout mS
Out_par:
   Data Cls_pcf , 1 , Out_adr , Out_default , 0%  ' passive device
Pwr_par:
   Data Cls_pcf , 2 , Pwr_adr , Pwr_default , 0%  ' passive device
Exp_par:
   Data Cls_pcf , 3 , Exp_adr , 0 , 1500%         ' auto sensor device


'---------------------------------------------------------------------------
Sub Pcfclass(byval Tag As Word , Byval Cmd As Byte , Byval Param As Word)
Local Pcf_index As Byte

   Pcf_index = Unit_ref(tag)                      ' get private ref

   Select Case Cmd
'----------------------------------------------------------------------------
'         SETUP Command
'----------------------------------------------------------------------------
   Case Cmd_setup:
      Call Unit_flg_set(tag , 1)                  ' set active
      Par_word = Loadlabel(pcfclass)
      Call Unit_vec_set(tag , Par_word)           'set vector
      Call Unit_par_set(tag , Param)              'set FLASH parameter
      Incr Pcfidx                                 'make private Index
      Call Unit_ref_set(tag , Pcfidx)             'store it
      Pcf_index = Pcfidx
      Pcf_bck(pcf_index) = Tag                    ' Backlink to unit
      Pcf_value(pcf_index) = Unit_parbyte(tag , 3)       ' default value
      Pcf_index = Unit_ref(tag)                   ' set received value
      I2cinit
      Par_word = Unit_parword(tag , 4)            ' refresh time
      If Par_word <> 0 Then
         Gosub Pcf_refresh_time                   ' sensor device
      Else
         Gosub Pcf_send_val                       ' act.device
      End If
'----------------------------------------------------------------------------
'         MESSAGE RECEIVED  Command
'----------------------------------------------------------------------------
   Case Cmd_msg:
      Tempb = Com_rdbyte()                        ' ignore source class
      Tempb = Com_rdbyte()                        ' ignore source ident
      Tempb = Com_rdbyte()                        ' data byte
      Pcf_value(pcf_index) = Tempb                ' store
      Gosub Pcf_send_val

'----------------------------------------------------------------------------
'         SET DEVICE Command
'----------------------------------------------------------------------------
   Case Cmd_set:
      Pcf_value(pcf_index) = Param                ' store
      Gosub Pcf_send_val

'----------------------------------------------------------------------------
'         TIME ELAPSE   Command
'----------------------------------------------------------------------------
   Case Cmd_time:
      Gosub Pcf_sens_val                          ' read PCF Device
'----------------------------------------------------------------------------
'send Message to host
      Call Com_txstart()
' write empfnger--------
      Tempb = 0
      Call Com_txbyte(tempb)                      ' dest class
      Call Com_txbyte(tempb)                      ' ident
' write absender---------
      Tempb = Unit_parbyte(tag , 0)               '
      Call Com_txbyte(tempb)                      ' source class
      Tempb = Unit_parbyte(tag , 1)               ' source ident
      Call Com_txbyte(tempb)
' write data-------------
      Tempb = Pcf_value(pcf_index)
      Call Com_txbyte(tempb)                      ' Data
' finish message---------
      Call Com_txclose()
'----------------------------------------------------------------------------

      Gosub Pcf_refresh_time                      ' refresh timer

   Case Else:

   End Select
   Exit Sub

Pcf_refresh_time:
   Par_word = Unit_parword(tag , 4)               ' refresh time
   Par_byte = Time_entry(0 , Tag , Cmd_time , Par_word)       'new timer
   Return
Pcf_send_val:
   Par_byte = Unit_parbyte(tag , 2)               ' device address
   Tempb = Pcf_value(pcf_index)
   I2cstart
   I2cwbyte Par_byte
   I2cwbyte Tempb
   I2cstop
   Tempb = 0
   Return
Pcf_sens_val:
   Par_byte = Unit_parbyte(tag , 2)               ' device address
   Incr Par_byte                                  ' I2C Read addr
   I2cstart
   I2cwbyte Par_byte
   I2crbyte Tempb , Nack
   I2cstop
   Pcf_value(pcf_index) = Tempb
   Return
End Sub





Adc_par0:
   Data Cls_adc , 0 , 0 , 0 , 500%                ' 500 mS refresh
Adc_par1:
   Data Cls_adc , 1 , 1 , 0 , 510%                ' 510 mS refresh
Adc_par2:
   Data Cls_adc , 2 , 2 , 0 , 490%                ' 490 mS refresh
Adc_par3:
   Data Cls_adc , 3 , 3 , 0 , 480%                ' 480 mS refresh
Adc_par4:
   Data Cls_adc , 4 , 4 , 0 , 2000%               ' 2 S refresh
Adc_par5:
   Data Cls_adc , 5 , 5 , 0 , 400%                ' 400 mS refresh
Adc_par6:
   Data Cls_adc , 6 , 6 , 0 , 512%                ' 512 mS refresh
Adc_par7:
   Data Cls_adc , 7 , 7 , 0 , 600%                ' 600 mS refresh


'---------------------------------------------------------------------------
Sub Adcclass(byval Tag As Word , Byval Cmd As Byte , Byval Param As Word)

   Select Case Cmd
'----------------------------------------------------------------------------
'         SETUP Command
'----------------------------------------------------------------------------
   Case Cmd_setup:
      Call Unit_flg_set(tag , 1)                  ' set active
      Par_word = Loadlabel(adcclass)
      Call Unit_vec_set(tag , Par_word)           'set vector
      Call Unit_par_set(tag , Param)              'set FLASH parameter

Config Adc = Single , Prescaler = Auto

      Start Adc

      Par_word = Unit_parword(tag , 4)            ' refresh time
      If Par_word <> 0 Then
         Gosub Adc_refresh_time                   ' sensor device
      End If
'----------------------------------------------------------------------------
'         TIME ELAPSE   Command
'----------------------------------------------------------------------------
   Case Cmd_time:
      Par_byte = Unit_parbyte(tag , 2)
      Par_word = Getadc(par_byte)
'----------------------------------------------------------------------------
'send Message to host
      Call Com_txstart()
' write empfnger--------
      Tempb = 0
      Call Com_txbyte(tempb)                      ' dest class
      Call Com_txbyte(tempb)                      ' ident
' write absender---------
      Tempb = Unit_parbyte(tag , 0)               '
      Call Com_txbyte(tempb)                      ' source class
      Tempb = Unit_parbyte(tag , 1)               ' source ident
      Call Com_txbyte(tempb)
' write data-------------
      Par_byte = Low(par_word)
      Call Com_txbyte(par_byte)                   ' Data LSB
      Par_byte = High(par_word)
      Call Com_txbyte(par_byte)                   ' Data MSB
' finish message---------
      Call Com_txclose()
'----------------------------------------------------------------------------

      Gosub Adc_refresh_time                      ' refresh timer

   Case Else:

   End Select
   Exit Sub

Adc_refresh_time:
   Par_word = Unit_parword(tag , 4)               ' refresh time
   Par_byte = Time_entry(0 , Tag , Cmd_time , Par_word)       'new timer
   Return

End Sub